// Flare.cpp: implementation of the CFlare class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "Kurushi.h"
#include "Flare.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

#include "vec3d.h"
#include "loadlum.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CFlare::CFlare(GLfloat p[3], float scale, float intens, float q_attenuation)
{
	shine_tic = 0;

	GLfloat red[3] = {1.0, 0.0, 0.0};
	GLfloat green[3] = {0.0, 1.0, 0.0};
	GLfloat blue[3] = {0.0, 0.0, 1.0};

	/* Shines */
	flare[0] = set_flare(blue);
	flare[1] = set_flare(green);
	flare[2] = set_flare(red);

	num_flares = 3;
	sc = scale;
	intensite = intens;
	q_att = q_attenuation;

	char filename[256];
	int i;
	GLuint id;
	glGenTextures(16, &id);

	glPixelStorei(GL_UNPACK_ALIGNMENT, 1);

	for (i = 0; i < 10; i++)
	{
		shineTex[i] = id;
		sprintf(filename, "Gfx\\Shine%d.bw", i);
		setup_texture(filename, shineTex[i]);
		id++;
	}

	vcopy(pos, p);
	AddLight();
}

CFlare::~CFlare()
{
	if (l)
	{
		delete l;
		l = NULL;
	}
}

Flare CFlare::set_flare(GLfloat color[3])
{
	Flare ret;

	ret.color[0] = color[0];
	ret.color[1] = color[1];
	ret.color[2] = color[2];

	return ret;
}

void CFlare::Draw(GLfloat from[3])
{
	GLfloat view_dir[3], tmp[3], dx[3], dy[3], dx_[3], dy_[3], y[3] = {0.0f, 1.0f, 0.0f};
	int i;

	/* view_dir = normalize(at-from) */
	vdiff(view_dir, pos, from);
	vnorm(view_dir);
	vcross(dx, view_dir, y);
	vnorm(dx);
	vcross(dy, dx, view_dir);
	vnorm(dy);
	vscale(dx, dx, sc);
	vscale(dy, dy, sc);
	vscale(dx_, dx, -0.5f);
	vscale(dy_, dy, -0.5f);

	glDepthFunc(GL_LEQUAL);
	glEnable(GL_BLEND);
	glBlendFunc(GL_ONE, GL_ONE);
	glEnable(GL_TEXTURE_2D);
	glDisable(GL_LIGHTING);

	for (i = 0; i < num_flares; i++)
	{
		glColor3fv(flare[i].color);
		glBindTexture(GL_TEXTURE_2D, shineTex[shine_tic]);
		shine_tic = (shine_tic + 1) % 10;

		vcopy(tmp, pos);
		vadd(tmp, tmp, dx_);
		vadd(tmp, tmp, dy_);

		glBegin(GL_QUADS);
		glTexCoord2f(0.0, 0.0);
		glVertex3fv(tmp);

		vadd(tmp, tmp, dx);
		glTexCoord2f(1.0, 0.0);
		glVertex3fv(tmp);

		vadd(tmp, tmp, dy);
		glTexCoord2f(1.0, 1.0);
		glVertex3fv(tmp);

		vadd(tmp, tmp, dx_);
		vadd(tmp, tmp, dx_);
		glTexCoord2f(0.0, 1.0);
		glVertex3fv(tmp);
		glEnd();
	}
	glDisable(GL_BLEND);
	glEnable(GL_LIGHTING);
}

void CFlare::setup_texture(char *filename, GLuint texobj)
{
	unsigned char *buf;
	int width, height, components;

	glBindTexture(GL_TEXTURE_2D, texobj);
	glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);					// verifier
	buf = load_luminance(filename, &width, &height, &components);
	gluBuild2DMipmaps(GL_TEXTURE_2D, 1, width, height, GL_LUMINANCE, GL_UNSIGNED_BYTE, buf);
	free(buf);
}

void CFlare::AddLight()
{
	GLfloat	ambientProperties[]  = {intensite, intensite, intensite, 1.0f};
	GLfloat	diffuseProperties[]  = {intensite, intensite, intensite, 1.0f};
	GLfloat	specularProperties[] = {0.0f, 0.0f, 0.0f, 1.0f};

	l = new CLight(pos);
	l->SetCarac(ambientProperties, diffuseProperties, specularProperties);

	glLightf(l->GetNum(), GL_QUADRATIC_ATTENUATION, q_att);
}

void CFlare::SetPos(GLfloat p [ 3 ])
{
	vcopy(pos, p);
	l->SetPos(p);
}
