// Light.cpp: implementation of the CLight class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "Kurushi.h"
#include "Light.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

GLint CLight::m_max_lights;
bool* CLight::m_used = NULL; 

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CLight::CLight(GLfloat *l)
{	
	if (!m_used)
	{
		glGetIntegerv(GL_MAX_LIGHTS, &m_max_lights);
		m_used = new bool[m_max_lights];
		for (int i = 0 ; i < m_max_lights ; i++)
			m_used[i] = false;
	}
	m_num = -1;
	m_valid = false;
	for (int i = 0 ; i < m_max_lights ; i++)
		if (!m_used[i])
		{
			m_num = i;
			m_valid = true;
			break;
		}
	if (m_valid)
	{
		m_used[m_num] = true;
		glEnable(GL_LIGHT0 + m_num);
	}
	m_pos[0] = l[0];
	m_pos[1] = l[1];
	m_pos[2] = l[2];
	m_pos[3] = l[3];
	RePos();
}

CLight::~CLight()
{
	if (m_valid)
	{
		glDisable(GL_LIGHT0 + m_num);
		m_used[m_num] = false;
		bool last = true;
		for (int i = 0 ; i < m_max_lights ; i++)
			if (m_used[i])
			{
				last = false;
				break;
			}
		if (last)
		{
			delete [] m_used;
			m_used = NULL;
		}
	}
}

void CLight::SetPos(GLfloat x, GLfloat y, GLfloat z)
{
	m_pos[0] = x;
	m_pos[1] = y;
	m_pos[2] = z;
	m_pos[3] = 1.0f;
	if (m_valid)
		glLightfv(GL_LIGHT0 + m_num, GL_POSITION, m_pos);
}

void CLight::SetPos(GLfloat *p)
{
	SetPos(p[0], p[1], p[2]);
}


void CLight::RePos()
{
	if (m_valid)
		glLightfv(GL_LIGHT0 + m_num, GL_POSITION, m_pos);
}

void CLight::Etat(bool etat)
{
	if (m_valid)
		if (etat)
			glEnable(GL_LIGHT0 + m_num);
		else
			glDisable(GL_LIGHT0 + m_num);
}

void CLight::SetCarac(GLfloat * ambient, GLfloat * diffuse, GLfloat * specular)
{
	if (m_valid)
	{
		glLightfv(GL_LIGHT0 + m_num, GL_AMBIENT, ambient);
		glLightfv(GL_LIGHT0 + m_num, GL_DIFFUSE, diffuse);
		glLightfv(GL_LIGHT0 + m_num, GL_SPECULAR, specular);
	}
}

