// RollingCube.cpp: implementation of the CRollingCube class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "Kurushi.h"
#include "RollingCube.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

#include "math.h"
#define PI 3.14159265f
#define NORMAL 0x0fffff

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CRollingCube::CRollingCube(int sizex, int sizez, float r, CString& texture_filename, float posx, float posy, float posz)
{
	m_r = r;
	m_sizex = sizex;
	m_sizez = sizez;
	m_t = 0;
	m_posx = posx;
	m_posy = posy;
	m_posz = posz; 

	c = new CCube(r, r/2, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, false, true, 255, texture_filename);
	m_option = new int*[sizex];			// j'espere que c'est correct
	for (int i = 0 ; i < sizex ; i++)
	{
		m_option[i] = new int[sizez];
		for (int j = 0 ;  j < sizez ; j++)
			m_option[i][j] = NORMAL;
	}
	m_v_speed = 75;
	m_v_disappear = 1000;
}

CRollingCube::~CRollingCube()
{
	if (c)
		delete c;
	for (int i = 0 ; i < m_sizex ; i++)
		delete [] m_option[i];
	delete [] m_option;
}

void CRollingCube::Draw()
{
	static float sqrt2 = float(sqrt(2));

	glEnable(GL_TEXTURE_2D);
	glColor4f(1.0f, 1.0f, 1.0f, 1.0f);

	float angle = (float(m_t)/m_v_speed);
	while (angle > 360)
		angle -= 360;
	float anglex = angle;
	while (anglex > 90)
		anglex -=90;
	int tour = m_t/m_v_speed/360;
	for (int i = 0 ; i < m_sizex ; i++)
		for (int j = 0 ; j < m_sizez ; j++)
			if (m_option[i][j] != 0)
			{
				GLfloat diffuse[4];
				diffuse[0] = diffuse[1] = diffuse[2] = float(m_option[i][j])/NORMAL;
				diffuse[3] = 1.0f;
				glMaterialfv(GL_FRONT, GL_DIFFUSE, diffuse);
				glPushMatrix();
					glTranslatef(i*m_r, 0, -j*m_r + (tour*4 + int(angle)/90)*m_r + m_posz);
					glRotatef(anglex,1,0,0);						// rotat avance
					glTranslatef(0, m_r/2, -m_r/2);
					glRotatef((int(angle)/90)*90.0f,1,0,0);			// rotat texture
					c->Draw(0,0,0); 
				glPopMatrix();
			}
	GLfloat diffuse[]  = { 0.8f, 0.8f, 0.8f, 1.0f };
    glMaterialfv(GL_FRONT, GL_DIFFUSE, diffuse);
}

void CRollingCube::Update(int t, CSol *s)
{
	m_t += t;
	for (int i = 0 ; i < m_sizex ; i++)
		for (int j = 0 ; j < m_sizez ; j++)
			if (m_option[i][j] != NORMAL && m_option[i][j] != 0)
			{
				m_option[i][j] -= m_v_disappear*t;
				if (m_option[i][j] < 0)
					m_option[i][j] = 0;
			}

	float offs = (int(m_t/m_v_speed/90)-s->z_offs)*m_r + m_posz;
	if (offs > 0)
		s->ReduceZ();
}

bool CRollingCube::Action(GLfloat x, GLfloat z)
{
	float offs = int(m_t/m_v_speed/90)*m_r + m_posz;
	return Action(int(x / m_r), int((offs - z) / m_r));
}

bool CRollingCube::Action(int x, int z)
{
	if (x < 0 || x >= m_sizex || z < 0 || z >= m_sizez)
		return false;
	if (m_option[x][z] == NORMAL)
		m_option[x][z] = NORMAL - 1;
	return true;
}

bool CRollingCube::Cube(GLfloat x, GLfloat z)
{
	float offs = int(m_t/m_v_speed/90)*m_r + m_posz;
	float i = x / m_r, j = (offs - z) / m_r;
	if (i < 0 || i >= m_sizex || j < 0 || j >= m_sizez)
		return false;
	return m_option[int(i)][int(j)] == NORMAL;	
}
