// Sol.cpp: implementation of the CSol class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "math.h"
#include "Kurushi.h"
#include "Sol.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

#define YMIN -5000

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CSol::CSol(int sizex, int sizey, int sizez, float r, CString& texture_filename, float posx, float posy, float posz)
{
	m_r = r;
	m_sizex = sizex;
	m_sizey = sizey;
	m_sizez = sizez;
	m_t = 0;
	m_posx = posx;
	m_posy = posy;
	m_posz = posz; 
	z_offs = 0;
	v_fall = 1;

	c = new CCube(r, r/2, -r/2, -r/2, 0, 0, 0, 0, 0, 0, 0, 0, 0, false, false, 0, texture_filename);
	m_option = new sol_option*[sizex];			// j'espere que c'est correct
	for (int i = 0 ; i < sizex ; i++)
	{
		m_option[i] = new sol_option[sizez];
		for (int j = 0 ;  j < sizez ; j++)
		{
			m_option[i][j].mode = NORMAL;
			m_option[i][j].option = 0;
		}
	}
}

CSol::~CSol()
{
	if (c)
		delete c;
	for (int i = 0 ; i < m_sizex ; i++)
		delete [] m_option[i];
	delete [] m_option;
}

void CSol::Draw()
{
	glPushMatrix();
		glTranslatef(m_posx, m_posy, m_posz);
		glEnable(GL_TEXTURE_2D);
		glColor4f(1.0f, 1.0f, 1.0f, 1.0f);
		int i,j;

		for (i = 0 ; i < m_sizex ; i++)
			for (j = 0 ; j < m_sizez ; j++)
				if (m_option[i][j].mode == NORMAL)
					c->Draw(i*m_r, 0, -j*m_r);
				else if (m_option[i][j].mode == FALLING)
					c->Draw(i*m_r, float(m_option[i][j].option)/1000, -j*m_r);
/*		for (i = 0 ; i < m_sizex ; i++)
			for (j = 1 ; j < m_sizey ; j++)
				c->Draw(i*m_r, -j*m_r, 0);
		for (i = 1 ; i < m_sizey ; i++)
			for (j = 1 ; j < m_sizez ; j++)
				c->Draw(m_r*(m_sizex-1), -i*m_r, -j*m_r);*/
		
		glDisable(GL_TEXTURE_2D);
		glDisable(GL_LIGHTING);
		glEnable(GL_BLEND);
		glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
		glEnable(GL_POLYGON_OFFSET_FILL);
		glPolygonOffset(-2.0, -1.0);			// ?
		
		for (int x = 0 ; x < m_sizex ; x++)
			for (int z = 0 ; z < m_sizez ; z++)
				if (m_option[x][z].mode == NORMAL && m_option[x][z].option)
				{
					glColor4ubv((unsigned char *)&m_option[x][z].option);
					glPushMatrix();
						glTranslatef(x * m_r, 0, -z * m_r);
						glBegin(GL_QUADS);
							glVertex3f(0.0f, 0.0f, 0.0f);
							glVertex3f( m_r, 0.0f, 0.0f);
							glVertex3f( m_r, 0.0f, -m_r);
							glVertex3f(0.0f, 0.0f, -m_r);
						glEnd();
					glPopMatrix();
				}
	glPopMatrix();
	glDisable(GL_BLEND);
	glEnable(GL_LIGHTING);
	glDisable(GL_POLYGON_OFFSET_FILL);

}

void CSol::Update(int t)
{
	m_t += t;
	for (int x = 0 ; x < m_sizex ; x++)
		for (int z = 0 ; z < m_sizez ; z++)
			if (m_option[x][z].mode == NORMAL)
			{
				if (m_option[x][z].option)
				{
					unsigned int s = ((unsigned int)((1 + sin(float(m_t)/500))*64)+64) << 24;
					m_option[x][z].option &= 0x00ffffff;
					m_option[x][z].option |= s;
				}
			}
			else if (m_option[x][z].mode == FALLING)
				if (m_option[x][z].option > YMIN)
				{
					m_option[x][z].option -= v_fall * t;
				}
				else
				{
					m_option[x][z].mode = DISABLED;
				}
}

bool CSol::Marque(float x, float z, int couleur)
{
	int i = int(x / m_r), j = int(-z / m_r);

	if (i < 0 || j < z_offs || i >= m_sizex || j >= m_sizez)
		return false;

	m_option[i][j].option = couleur;
	return true;
}

void CSol::Action(CJoueur *j, CRollingCube * r)
{
	for (int x = 0 ; x < m_sizex ; x++)
		for (int z = 0 ; z < m_sizez ; z++)
			if (m_option[x][z].option /* == j->couleur*/)
			{
				m_option[x][z].option = 0;
				r->Action(GLfloat(x*m_r), GLfloat(-z*m_r));
			}
}

void CSol::ReduceZ()
{
	for (int x = 0 ; x < m_sizex ; x++)
	{
		m_option[x][z_offs].mode = FALLING;
		m_option[x][z_offs].option = -1;
	}
	if (z_offs < m_sizez)
		z_offs++;
}
